require 'zip'
require 'rexml/document'
require 'Bookmark'
require 'Utils'


# This class extracts bookmarks (toc) from an EPUB file
class EpubExtractor

  EPUB_FILE = 'files/Das Paket_ Psychothriller - Fitzek, Sebastian.epub'
  
  # The initialization method
  def initialize( options )
    @options = options
  end
  
  include Utils
  
  # Extracts the bookmark (toc) info from epub file
  def extract( from )
    to = change_extension( result, '.xml' )
    Zip::File.open( from ) do | zip_file |
      begin 
        zip_file.extract('OEBPS/toc.ncx', to) { true }      
      rescue
        zip_file.extract('toc.ncx', to) { true }      
      end
    end
    to
  end
  
  # Parses a xml element node recursively and adds children to the parent bookmark
  def parse_node( bookmark, nav_point )
    nav_point.elements.each( 'navPoint' ) do | nav_point_child |
      title = nav_point_child.elements()['navLabel/text'].text
      title = strip_from_title( title )                                   # strip from the beginning
      child = Bookmark.new(title, bookmark.level + 1, bookmark)
      if (bookmark.level() == 1)
        @level2_number += 1
        child.number = @level2_number
      end
      bookmark.children.push( child )
      
      parse_node( child, nav_point_child )
    end
  end
  
  # Parses the extracted xml toc file
  def parse( from )
    root = Bookmark.new( "Root" )
    @level2_number = 0

    xml_text = ''
    File.open( from ) do | file |
      xml_text = file.read
    end
    xml_document = REXML::Document.new( xml_text )
    
    xml_document.elements.each( '/ncx/navMap' ) do | nav_map |
      parse_node( root, nav_map )
    end
    
    root
  end

end
